/*
 * Created on Oct 17, 2003
 *
 */
package com.punosmusic.larrybirthday;

import com.softsynth.jmsl.MusicShape;

/**
 * Input is a MusicShape and a dimension
 * Output is a 1-dimensional MusicShape with 1, 0, -1 values indicating direction of 
 * all combinatorial pairs
 *  
 * @author Nick Didkovsky, (c) 2003 Nick Didkovsky, didkovn@mail.rockefeller.edu, All rights reserved
 *
 */
public class CombinatorialContourAnalyzer {

	/** @return true if element s1[pos] has the same contour relationship to all elements in s1 as
	 * s2[pos] has to all elements in s2 
	 */
	public static boolean contourPositionsEqual(String s1, String s2, int pos) {
		boolean equal = true;
		char c1 = s1.charAt(pos);
		char c2 = s2.charAt(pos);
		for (int i = 0; i < Math.min(s1.length(), s2.length()); i++) {
			if (comparisonValue(c1, s1.charAt(i)) != comparisonValue(c2, s2.charAt(i))) {
				equal = false;
				break;
			}
		}
		return equal;
	}

	/** @return true if char element s1[pos] has the same contour relationship to all elements in String s1 as
	* double element s[pos, dim] has to all elements in MusicShape s along dimension dim 
	*/
	public static boolean contourPositionsEqual(String s1, MusicShape s, int pos, int dim) {
		boolean equal = true;
		char c1 = s1.charAt(pos);
		double v2 = s.get(pos, dim);
		for (int i = 0; i < Math.min(s1.length(), s.size()); i++) {
			if (comparisonValue(c1, s1.charAt(i)) != comparisonValue(v2, s.get(i, dim))) {
				equal = false;
				break;
			}
		}
		return equal;
	}

	public static int comparisonValue(char c1, char c2) {
		if (c1 == c2)
			return 0;
		else if (c1 > c2)
			return -1;
		else
			return 1;
	}

	public static int comparisonValue(double v1, double v2) {
		if (v1 == v2)
			return 0;
		else if (v1 > v2)
			return -1;
		else
			return 1;
	}

	/** @return 1-dimensional MusicShape with 1, 0, -1 values indicating direction of 
	 * all combinatorial pairs.
	 * */
	public static MusicShape getOneDimensionalContour(MusicShape musicShape, int dimension) {
		MusicShape contourShape = new MusicShape(1);
		contourShape.setDimensionName(0, "direction");
		for (int i = 0; i < musicShape.size() - 1; i++) {
			double outerValue = musicShape.get(i, dimension);
			for (int j = i + 1; j < musicShape.size(); j++) {
				double innerValue = musicShape.get(j, dimension);
				if (outerValue < innerValue) {
					contourShape.add(1);
				}
				else if (outerValue == innerValue) {
					contourShape.add(0);
				}
				else {
					contourShape.add(-1);
				}
			}
		}
		return contourShape;
	}

	/** @return NxN MusicShape with 1, 0, -1 values indicating direction of 
	 * all combinatorial pairs, where N = musicShape.dimension()
	 * Each dimension n of the resulting contour shape corresponds to the combinatorial contour 
	 * relationship between element n and all other elements.
	 * */
	public static MusicShape getNxNContour(MusicShape musicShape, int dimension) {
		MusicShape contourShape = new MusicShape(musicShape.size());
		for (int i = 0; i < musicShape.size(); i++) {
			double[] dar = new double[contourShape.dimension()];
			contourShape.add(dar);
		}

		for (int dimOfContourShape = 0; dimOfContourShape < musicShape.size(); dimOfContourShape++) {
			double valueToCompare = musicShape.get(dimOfContourShape, dimension);
			for (int i = 0; i < musicShape.size(); i++) {
				double value = musicShape.get(i, dimension);
				double direction = comparisonValue(valueToCompare, value);
				contourShape.set(direction, i, dimOfContourShape);
			}
		}
		return contourShape;
	}

	/** @return 1-dimensional MusicShape with 1, 0, -1 values indicating direction of 
	 * all combinatorial pairs of characters.
	 * */
	public static MusicShape getOneDimensionalContour(String str) {
		return getOneDimensionalContour(StringToMusicShape(str), 0);
	}

	/** @return NxN MusicShape with 1, 0, -1 values indicating direction of 
	 * all combinatorial pairs, where N = musicShape.dimension()
	 * Each dimension n of the resulting contour shape corresponds to the combinatorial contour 
	 * relationship between element n and all other elements.
	 * */
	public static MusicShape getNxNContour(String str) {
		return getNxNContour(StringToMusicShape(str), 0);
	}

	private static MusicShape StringToMusicShape(String str) {
		MusicShape temp = new MusicShape(1);
		for (int i = 0; i < str.length(); i++) {
			int ch = (int) str.charAt(i);
			temp.add(ch);
		}
		return temp;
	}

	public static void main(String[] args) {
		MusicShape source = new MusicShape(1);
		source.add(10);
		source.add(20);
		source.add(10);
		source.add(30);
		source.print();
		MusicShape contour = CombinatorialContourAnalyzer.getNxNContour(source, 0);
		contour.print();
		String sudString = ContourToString.translate(contour);
		System.out.println(sudString);
		System.out.println("============================");

		String msg = "ABAD";
		contour = CombinatorialContourAnalyzer.getNxNContour(msg);
		contour.print();
		sudString = ContourToString.translate(contour);
		System.out.println(sudString);

	}
}
